// Tajmer_508.C           12F629 / InterniRC 4Mhz    ++HARDWARE rev1.0
// prototip: PIC16F84A - 4Mhz
// Kompajler: mikroC ver2.1.0.0 (http://www.mikroelektronika.co.yu
// Autor:  Nebojsa Pejcic,
//          http://epraktikum.co.yu
//          mailto: npejcic@epraktikum.co.yu

// (*)  Kratak Stisak = +> NON_STOP mode -> zatim 2h -> 1h... -> 2min -> >-+
//                      +--------------------------------------------------+
// (*)  Svaki sledeci pritisak podesavamo tajminge koje indiciraju LED
// (*)  Duzim drzanjem gasimo uredjaje
// (**) Okrenut redosled paljenja LEDICA za prikaz moda

// start: 05.07.2002.

// modovi : Info_LED blinka - OFF unlimited
//          Info_LED svetli - ON          // o => svetli  - => ne_svetli
//
//   MODE       TRAJANJE        LEDICE       Redosled ukljucenja
//  mode 0     2   minuta      0b000   ooo             7  +>-+
//  mode 1     10  min         0b001   oo-             6    
//  mode 2     30  min         0b010   o-o             5     duzi stisak
//  mode 3     45  min         0b011   o--             4    +-----------+
//  mode 4     60  min         0b100   -oo             3        OFF    
//  mode 5     90  min         0b101   -o-             2               
//  mode 6     120 min         0b110   --o             1  <--+           
//  mode 7   neograniceno ON   0b111   ---       Off pa kratak stisak    
//  mode 8   neograniceno OFF  0b111   ---       duzi stisak   <---------+

//  Duzi stisak tastera - OFF
//  Kraci menjamo stisak tastera - ON, mode change
//  NAPOMENA: U ovoj verziji nema interne kalibracije oscilatora!

// PIC12C508A
#define TASTER   GPIO.F3  //@  PORTBIT(GPIO, 3); // Ulazni pin Tastera
#define LED_0    GPIO.F0  //@  PORTBIT(GPIO, 0); // Izlazni pin LEDICA0
#define LED_1    GPIO.F1  //@  PORTBIT(GPIO, 1); // Izlazni pin LEDICA1
#define LED_2    GPIO.F2  //@  PORTBIT(GPIO, 2); // Izlazni pin LEDICA2
#define LED_info GPIO.F4  //@  PORTBIT(GPIO, 4); // Izlazni pin LED_info
#define RELE     GPIO.F5  //@  PORTBIT(GPIO, 5); // Izlazni pin RELEJ

const unsigned char prikaz[9]=  // pazi!!!  '0' svetli '1' ne svetli
{
  0b00000000,     // mode 0    // 2   minuta
  0b00000001,     // mode 1    // 10  min
  0b00000010,     // mode 2    // 30  min
  0b00000011,     // mode 3    // 45  min
  0b00000100,     // mode 4    // 60  min
  0b00000101,     // mode 5    // 90  min
  0b00000110,     // mode 6    // 120 min
  0b00000111,     // mode 7 -  neograniceno ON
  0b00000111      // mode 8 -  neograniceno OFF
};

//#define MINUTA(x) ((x)*1172) // 256*200*1172 = 60006400uS <= ovoliko traje jedan minut
#define TRAJANJE_MINUT  1172   // 256*200*1172 = 60006400uS <= ovoliko traje jedan minut

const unsigned char trajanje[9]=
{
  2,   //2344,   // 2   minuta        // mode 0 -   2  min
  10,  //23440,  // 10  min           // mode 1 -  10  min
  30,  //35160,  // 30  min           // mode 2 -  30  min
  45,  //52740,  // 45  min           // mode 3 -  45  min
  60,  //70320,  // 60  min           // mode 4 -  60  min
  90,  //105480, // 90  min           // mode 5 -  90  min
  120, //140640, // 120 min           // mode 6 -  120 min
  10,  //11720,  // nebitno           // mode 7 -  neograniceno ON
  10   //11720,  // nebitno           // mode 8 -  neograniceno OFF
};

unsigned char count,count1;
unsigned long count2;
signed char mode;
unsigned char drzim_taster;

// BIT Definitions
unsigned char flegovi;
#define samo_jednom   flegovi.F0
#define zmirko        flegovi.F1

void main()
{
//  asm
//  {
//    bsf STATUS, RP0
//    call 3ffh
//    movwf OSCCAL
//    bcf STATUS, RP0
//  };

// PIC12F629
  OPTION_REG = 0b11010111; // 1:256 tmr0 prescaler, pullups disabled, prescaller=TMR0
  TRISIO     = 0b11001000;  // Svi pinovi GPIO su izlazni osim GPIO3!!
  VRCON      = 0b00000000;  // Voltage reference
  CMCON      = 0b00000111;  // Comparatori iskljuceni
  WPU        = 0b00000000;
  IOCB       = 0b00000000;

  INTCON     = 0b00000000; // pogasi sve interapte
  PIE1       = 0b00000000; // ukljuci samo TMR1 interapt

  // init
  RELE = 0;
  mode = 8;  // default mode // nakon dolaska struje

  // timer increment 256uS
  // sinhronizacija
  TMR0 = 0;
  GPIO = 0b00000000;

  for(count=0; count<=39; count++) //40*51200uS = 2 sek
  {
    while(TMR0 < 199); // stabilisi se po dolasku napona
    TMR0 = 0;
  }

  while(1) //Stay in this loop
  {
    // ide na svakih 51200 uS
    while(TMR0 < 199);
    TMR0 = 0;

    // ########## Tajmerske zavrzlame
    count1++;

    // modovi rada
    if(mode <  7) count2++;
    if(mode == 8) RELE = 0;
    if(mode == 7) RELE = 1;

    if(count1 > 20) // 0.5=10 sekundi   1sek=20
    {         
      count1 = 0;
      if(mode == 8)
      {
        //if(zmirko) zmirko = 0; else zmirko = 0;
        if(zmirko) zmirko = 0; else zmirko = 1;
        LED_info = zmirko;
      }
      else
      {
        LED_info = 0; // blinkaj ili svetli
      }
    }

    if(count2 > ((unsigned long)trajanje[mode]*TRAJANJE_MINUT))  // menjamo vreme trajanja
    {
      count2 = 0;
      mode = 8;
    }

    // ########## Tastaturske zavrzlame
    if(TASTER == 0) // taster
    {
      drzim_taster++;        // da detektujemo duzinu stiska
      count2 = 0;            // cim pritisnes taster broji ispocetka
      if(drzim_taster > 40)  // ako drzim taster (40 = aprox) 2 sekunde
      {
        mode = 8;            // gasi sve i stavi se u mod cekanja
      }
      if(samo_jednom)        // ako je samo jednom pritisnut
      {
        mode--;     // idemo sledeci mod, i to unatraske 2h,1h,...2min
        if(mode < 0) mode = 7; // ako je mod manji od 0 idemo na mod 7 i tako..
        samo_jednom = 0;     // sada moramo da pustimo taster za sledeci mod
      }                      //            ^
    }                        //            
    else                     //   Debounce 
    {                        //            v
      samo_jednom  = 1;      // ok pustili smo taster, sada moze sledeci stisak
      drzim_taster = 0;      //
    }

    // ######## Prikaz informacija
    //PIC12F629
    GPIO |= 0b00000111;     // pogasimo sve info LED-ice
    GPIO &= (prikaz[mode] | 0b11111000); // okrecemo logiku zbog
         //samo nalepimo stanje (mod) na       // vezivanja LEDica na +Vcc
  }
}






